@@ -1,4 +1,4 @@
-# Copyright 2018-2020 Faculty Science Limited
+# Copyright 2018-2021 Faculty Science Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,28 +17,64 @@ Manage the Faculty cluster configuration.
 """
 
 
-from collections import namedtuple
-
+from attr import attrs, attrib
 from marshmallow import fields, post_load
 
 from faculty.clients.base import BaseSchema, BaseClient
 
 
-NodeType = namedtuple(
-    "NodeType",
-    [
-        "id",
-        "name",
-        "instance_group",
-        "max_interactive_instances",
-        "max_job_instances",
-        "milli_cpus",
-        "memory_mb",
-        "num_gpus",
-        "gpu_name",
-        "cost_usd_per_hour",
-    ],
-)
+@attrs
+class NodeType(object):
+    """A single tenanted node type in the platform.
+
+    Parameters
+    ----------
+    id : str
+        A unique identifier for the node type, usually the cloud provider's
+        machine type.
+    name : str
+        A short descriptive name for the node type, usually the same as the id.
+    instance_group : str
+        The Kubernetes instance group that provides this node type.
+    max_interactive_instances : int
+        The maximum number of instances of this node type that can be used for
+        workspace servers, apps and APIs at any given time.
+    max_job_instances : int
+        The maximum number of instances of this node type that can be used for
+        jobs at any given time.
+    milli_cpus : int
+        The amount of CPU resource available to servers running on nodes of
+        this type.
+    memory_mb : int
+        The amount of memory available to servers running on nodes of this
+        type.
+    num_gpus : int
+        The number of GPUs available to servers running on nodes of this type.
+    gpu_name : str, optional
+        A descriptive name of the type of GPUs available on this node type,
+        when available.
+    cost_usd_per_hour : decimal.Decimal
+        The on-demand hourly cost for this node type, in USD.
+    spot_max_usd_per_hour : decimal.Decimal, optional
+        The bid price set for this node type, when using spot instances. The
+        actual cost will usually be lower than this, but the bid price sets the
+        maximum cost (if the spot price increases beyond this price, AWS will
+        shut the instances down). If unset, indicates that the node type is
+        configured to run with on demand instances, and the `cost_usd_per_hour`
+        will be charged.
+    """
+
+    id = attrib()
+    name = attrib()
+    instance_group = attrib()
+    max_interactive_instances = attrib()
+    max_job_instances = attrib()
+    milli_cpus = attrib()
+    memory_mb = attrib()
+    num_gpus = attrib()
+    gpu_name = attrib()
+    cost_usd_per_hour = attrib()
+    spot_max_usd_per_hour = attrib()
 
 
 class ClusterClient(BaseClient):
@@ -103,6 +139,7 @@ class ClusterClient(BaseClient):
         instance_group,
         max_interactive_instances,
         max_job_instances,
+        spot_max_usd_per_hour=None,
     ):
         """Configure a single tenanted node type on the cluster.
 
@@ -125,12 +162,20 @@ class ClusterClient(BaseClient):
         max_job_instances : int
             The maximum number of jobs runs using this node type to allow to
             run simultaneously.
+        spot_max_usd_per_hour : decimal.Decimal, optional
+            The bid price in USD, when this instance group is configured to run
+            on spot instances.
         """
         payload = {
             "name": name,
             "instanceGroup": instance_group,
             "maxInteractiveInstances": max_interactive_instances,
             "maxJobInstances": max_job_instances,
+            "spotMaxUsdPerHour": (
+                None
+                if spot_max_usd_per_hour is None
+                else str(spot_max_usd_per_hour)
+            ),
         }
         self._put_raw(
             "/node-type/single-tenanted/{}/configuration".format(node_type_id),
@@ -169,6 +214,9 @@ class _NodeTypeSchema(BaseSchema):
     cost_usd_per_hour = fields.Decimal(
         data_key="costUsdPerHour", required=True
     )
+    spot_max_usd_per_hour = fields.Decimal(
+        data_key="spotMaxUsdPerHour", missing=None
+    )
 
     @post_load
     def make_node_type(self, data, **kwargs):
