@@ -1,4 +1,4 @@
-# Copyright 2018-2020 Faculty Science Limited
+# Copyright 2018-2021 Faculty Science Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -134,8 +134,7 @@ def test_model_client_list(mocker):
 
     schema_mock.assert_called_once_with(many=True)
     ModelClient._get.assert_called_once_with(
-        "/project/{}/model".format(PROJECT_ID, MODEL_ID),
-        schema_mock.return_value,
+        "/project/{}/model".format(PROJECT_ID), schema_mock.return_value
     )
 
 
